# -*- coding: latin-1 -*-
'''
pluginConfig.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
# Import sapyto
import core.sapytoCore as core
import core.output.outputManager as om
from core.sapytoException import sapytoException
from core.ui.consoleUi.consoleMenu import consoleMenu

# for parsing plugin options
import xml.dom.minidom

class pluginConfig(consoleMenu):
	'''
	This class is a menu for configuring plugin internal options.
	
	@author: Andres Riancho ( andres.riancho@gmail.com )	
	'''
	def __init__( self, sapyto, commands=[] ):
		consoleMenu.__init__(self)
		self._menu = {'help':self._help, 'set':self._set,'view':self._view,'back':self._back}
		self._sapyto = sapyto
		self._commands = commands
		self._Options = {}
		self._pluginName = ''
	
	def sh( self, pluginName ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		# First we need to get a plugin instance and set one or two parameters
		try:
			self._pluginInstance = self._sapyto.getPluginInstance( pluginName )
		except sapytoException, e:
			om.out.console('Error: ' + str(e) )
		else:
			self.parseXML( self._pluginInstance.getOptionsXML() )
			self._pluginName = pluginName
			
			prompt = 'sapyto/plugin/' + pluginName + '>>> '
			self._mainloop( prompt )

		
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command != '':
			if command in self._menu.keys():
				func = self._menu[command]
				return func(parameters)
			else:
				om.out.console( 'command not found' )
				return True

	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		if len( parameters ) == 0:
			om.out.console('The following commands are available:')
			self.mprint('help','You are here. help [command] prints more specific help.')
			self.mprint('set' ,'Set a parameter value.')
			self.mprint('view','List all configuration parameters and current values.')
			self.mprint('back',	'Return to previous menu.')			

		else:
			if len( parameters ) == 1:
				if parameters[0] in self._menu.keys():
					if parameters[0] == 'set':
						om.out.console( 'Set a parameter value.')
						om.out.console( 'Sintax: set {parameter} {value}')
					elif parameters[0] == 'list':
						om.out.console( 'List all configuration parameters and current values.')
						om.out.console( 'Sintax: list')
		
		return True
	
	def _view( self , parameters):
		'''
		List all configurable parameters of a plugin.
		'''
		self.mprint3('Parameter','Value', 'Description')
		self.mprint3('=========','=====', '===========')
		for value in self._Options.keys():
			self.mprint3(value,self._Options[value]['default'],self._Options[value]['desc'])
		return True
		
	
	def _set( self , parameters):
		'''
		Configure a value.
		'''
		if len( parameters ) < 2:
			om.out.console('Invalid call to set, please see the help:')
			self._help( ['set'] )
		else:
			param = parameters[0]
			value = parameters[1:]
			value = ' '.join( value )
			if param in self._Options.keys():
				type = self._Options[param]['type']
				try:
					core.sCore.parseOption(type, value)
				except sapytoException:
					om.out.console( 'Parameter ' + param + ' must be of type ' +  type)
				else:
					self._Options[ param ]['default'] = value
			else:	
				om.out.console( 'Unknown parameter.' )
		return True
		
	def _back( self, parameters ):
		'''
		Calls plugin.setOptions(...) and then goes back.
		'''
		options4plugin = {}
		for param in self._Options.keys():
			type = self._Options[param]['type']
			value = self._Options[param]['default']
			try:
				resVal = core.sCore.parseOption(type, value)
			except sapytoException:
				om.out.console( 'Parameter ' + param + ' must be of type ' +  type)
			else:
				options4plugin[ param ] = resVal
		#FIXME: Pass options better!
		self._sapyto.setPluginOptions( self._pluginName, options4plugin )
		om.out.setPluginOptions( self._pluginName, options4plugin )
		return False
