# -*- coding: latin-1 -*-
'''
gwmon.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
from core.basePlugins.baseShellPlugin import baseShellPlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *
from core.misc import *
import core.sapytoCore as core
import plugins.audit.checkGwmon as audit
from core.exceptions.dontRunException import dontRunException
import re
import struct

# We need 'raw' connections
import socket
import copy


class gwmon(baseShellPlugin):
	'''
	Implements a Gateway Monitor client. Get useful information from SAP GW.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		baseShellPlugin.__init__(self)
		self._menu = {'help':self._help, 'connect':self._connect, 'disconnect':self._disconnect, 'getClients':self._getClients, 'getInfo':self._getInfo, 'getSecurityInfo':self._security, 'shutdown':self._shutdown, 'set':self._set, 'view':self._view, 'back':self._back}
		self._Options = {}
		self._access = False
		self._gwhost = 'Current target'
		self._gwserv = 'sapgw00'
		self._checkFirst = 1
		
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/audit/gwmon>>> '
		self.parseXML(self.getOptionsXML())
		self._mainloop( prompt )		
	
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command != '':
			if command in self._menu.keys():
				func = self._menu[command]
				try:
					res = func(parameters)
				except sapytoException,e:
					om.out.console( str(e) )
				else:
					return res
			else:
				om.out.console( 'command not found' )
		return True
		
	
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		om.out.console('The following commands are available:')
		self.mprint('help','You are here.')
		self.mprint('connect','Connect to target.')
		self.mprint('getClients','Get connected clients (useful to get tpnames).')
		self.mprint('getSecurityInfo','Get the content of reg_info and sec_info.')
		self.mprint('getInfo','Get information about SAP Gateway.')
		self.mprint('shutdown','Remotely shutdown SAP Gateway.')
		self.mprint('set','Set plugin options.')
		self.mprint('view','View plugin options.')
		self.mprint('add','Add parameters and tables.')
		self.mprint('disconnect','Close connection with SAP GW.')
		self.mprint('back','Return to previous menu.')
		return True
	
	def _back( self, parameters ):
		return False

	def run( self ):
		try:
			core.sCore.validateTargets(self._targets)
		except dontRunException:
			return False		
		self.sh()
	
	def _connect(self, parameters):
		if self._gwhost == 'Current target':
			self._gw = self._conn.ashost
		else:
			self._gw = self._gwhost
		
		# Create the socket to connect with GW
		self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		try:
			self._socket.connect((self._gw, socket.getservbyname(self._gwserv)))
		except socket.error, msg:
			om.out.information('Couldn\'t connect with remote Gateway.')
		else:
			om.out.information('Trying to access Gateway Monitor...')
			if self._checkFirst == 1:
				om.out.information('(checkFirst enabled. Running \'' + audit.__name__.split('.')[-1] + '\' plugin first...)')
				if not kb.kb.getData('checkGwmon', 'remoteMonitor') :
					au = audit.checkGwmon()
					au.setConn(copy.deepcopy(self._conn))
					# Set Options for checkGwmon, UGLY!!! 
					au.setGwhost(self._gwhost)
					au.setGwserv(self._gwserv)
					om.out.quiet()
					au.run()
					om.out.talk()
			
			self._access = kb.kb.getData('checkGwmon', 'remoteMonitor') 
			if self._access:
				om.out.information('Access granted.')
			else:
				om.out.information('Access denied.')
		
			return True
	
	def _getInfo(self, parameters):
		'''
		Get lot of useful information from SAP Gateway (kernel version, patches, etc).
		'''
		if self._access:
			# Get info
			paq = '\x00'*3 + '\x88\x03\x09' + '\x23\x00\x00\x00\x05\x02' + '\x00'*138
			try:
				self._socket.send(paq)
			except socket.error, msg:
				om.out.information('Couldn\'t send data: Try re-connecting.' )
				return True
			# Receive packet length
			size = self._socket.recv(4)
			try:
				size = struct.unpack('!L', size)[0]
				# Receive the rest
				data = self._socket.recv(size)
				om.out.information('Gateway release information:')
				om.out.information(data)
			except:
					om.out.information('Error in connection. Retry.')
		else:
			om.out.information('Not connected.')

	def _shutdown(self, parameters):
		'''
		Get lot of useful information from SAP Gateway (kernel version, patches, etc).
		'''
		if self._access:
			paq = '\x00'*3 + '\x88\x03\x09' + '\x20\x00\x01\x00\xa2\x96\x34\x7c' + '\x00'*126
			try:
				self._socket.send(paq)
			except socket.error, msg:
				om.out.information('Couldn\'t send data: Try re-connecting.' )
				return True
			om.out.information('Shutdown command sent.')
		else:
			om.out.information('Not connected.')
	
	def _security(self, parameters):
		'''
		Get security information (reg_info, sec_info).
		'''
		paqs = [{'FILE':'reg_info','PAQ':'\x00'*3 + '\x88\x03\x09' + '\x23\x00\x00\x00\x0e\x02\xd1\x00\x6d\x0a' + '\x00'*124},]
		paqs.append({'FILE':'sec_info','PAQ':'\x00'*3 + '\x88\x03\x09' + '\x23\x00\x00\x00\x0f\x02\xd1\x00\x6d\x0a' + '\x00'*124})
		
		if self._access:
			for paq in paqs:
				try:
					self._socket.send(paq['PAQ'])
				except socket.error, msg:
					om.out.information('Couldn\'t send data: Try re-connecting.' )
					return True
				# Receive packet length
				size = self._socket.recv(4)
				try:
					size = struct.unpack('!L', size)[0]
					# Receive the rest
					data = self._socket.recv(size)
					om.out.information('Contents of ' + paq['FILE'] + ' file:')
					om.out.information(data)
				except:
					om.out.information('Error in connection. Retry.')
			else:
				om.out.information('Not connected.')
	
	
	def _getClients(self, parameters):
		'''
		Get registered clients at SAP Gateway. The same as using gwmon->3.
		'''
		def parseResClient(data):
			'''
			Parse the registered client data received.
			@parameter data: Raw data from gw.
			@return: List containing registered clients' information.
			'''
			res = []
			header = '\x00\x03\x01\x1e\x01'
			# Yes, I keep being a regex dummy... :(
			data += header
			fields = [f for f in data.split(header) if f != '']
			for field in fields:
					# Get the ip
					ip = socket.inet_ntoa(re.findall('.{15}(.{4})', field, re.DOTALL)[0])
					# Get the LU (hostname)
					host = re.findall('\xff\xff.{4}(.*?)\x00', field, re.DOTALL)[0]
					# TPname (the important data!)
					tpname = re.findall(host + '\x00*(.*?)\x00', field, re.DOTALL)[0]
					date =  re.findall(tpname + '\x00*(.*?)\x00', field, re.DOTALL)[0]
					res.append({'TPNAME':tpname, 'IP':ip, 'HOST':host, 'DATE':date})
			return res

		if self._access:
			# Get clients
			#paq = '\x00'*3 + '\x88\x03\x09' + '\x34\xff\xff\x7c\xc4\x13\x39\x7c\x08\xb5\x38' + '\x00'*123
			paq = '\x00'*3 + '\x88\x03\x09' + '\x34\xff\xff\x00\x00\x00\x00\x00\x00\xff\xff' + '\x00'*123
			try:
				self._socket.send(paq)
			except socket.error, msg:
				om.out.information('Couldn\'t send data: Try re-connecting.' )
				return True
			# Receive packet length
			size = self._socket.recv(4)
			try:
				size = struct.unpack('!L', size)[0]
				# Receive the rest
				data = self._socket.recv(size)
				# Parse the response and print results
				res = parseResClient(data)
				for l in res:
					om.out.information('Tpname: ' + l['TPNAME'] + '\tHost: ' + l['HOST'] + '\tIP: ' + l['IP']  + '\tDate: ' + l['DATE']) 
			except:
					om.out.information('Error in connection. Retry.')
		else:
			om.out.information('Not connected.')
	
	
	def _disconnect(self, parameters):
		if self._conn.is_connected():
			self._conn.close()
		om.out.information('Disconnected.')
		return True

	def _view( self , parameters):
		'''
		List all configurable parameters of a plugin.
		'''
		self.mprint('Parameter','Value')
		self.mprint('=========','=====')
		for value in self._Options.keys():
			self.mprint(value,self._Options[value]['default'])
		return True
	
	def _set( self , parameters):
		'''
		Configure a value.
		'''
		if len( parameters ) < 2:
			om.out.console('Invalid call to set, please see the help:')
			self._help( ['set'] )
		else:
			param = parameters[0]
			value = parameters[1:]
			value = ' '.join( value )
			resVal = None
			if param in self._Options.keys():
				type = self._Options[param]['type']
				try:
					resVal = core.sCore.parseOption(type, value)
				except sapytoException:
					om.out.console( 'Parameter ' + param + ' must be of type ' +  type)
				else:
					self._Options[ param ]['default'] = resVal
			else:	
				om.out.console( 'Unknown parameter.' )
		return True
		
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="gwhost">\
				<default>Current target</default>\
				<desc>Gateway host</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
			<Option name="gwserv">\
				<default>sapgw00</default>\
				<desc>Gateway service</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
			<Option name="checkFirst">\
				<default>1</default>\
				<desc>Checks accessibility of Gateway Monitor first.</desc>\
				<type>Integer</type>\
			</Option>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'gwhost' in OptionList.keys(): 
			self._gwhost = OptionList['gwhost']
		if 'gwserv' in OptionList.keys(): 
			self._gwserv = OptionList['gwserv']
		if 'checkFirst' in OptionList.keys(): 
			self._checkFirst = OptionList['checkFirst']
