# -*- coding: latin-1 -*-
'''
stick.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
from core.basePlugins.baseShellPlugin import baseShellPlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *
from core.exceptions.dontRunException import dontRunException
from core.misc import *
import core.sapytoCore as core


class stick(baseShellPlugin):
	'''
	Sticks to the RFC Server. Usefull for raw requests, blocking connections, etc.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		baseShellPlugin.__init__(self)
		self._menu = {'help':self._help, 'connect':self._connect, 'disconnect':self._disconnect, 'set':self._set, 'view':self._view, 'add':self._add, 'call':self._call, 'back':self._back}
		self._Options = {}
		self._funcname = ''
		self._params = []
		self._tables = []
		self._resparams = []
		self._restables = []
	
	def sh( self ):
		'''
		Starts the shell's main loop.
		
		@return: The prompt
		'''
		prompt = 'sapyto/attack/stick>>> '
		self.parseXML(self.getOptionsXML())
		self._mainloop( prompt )		
	
	def _exec( self, command ):
		'''
		Executes a user input.
		'''
		command, parameters = self._parse( command )
		
		if command != '':
			if command in self._menu.keys():
				func = self._menu[command]
				try:
					res = func(parameters)
				except sapytoException,e:
					om.out.console( str(e) )
				else:
					return res
			else:
				om.out.console( 'command not found' )
			return True
		
	
	def _help( self, parameters ):
		'''
		Prints a help message to the user.
		'''
		om.out.console('The following commands are available:')
		self.mprint('help','You are here.')
		self.mprint('connect','Connect to target.')
		self.mprint('set','Set plugin options.')
		self.mprint('view','View plugin options.')
		self.mprint('add','Add parameters and tables.')
		self.mprint('call','Perform a raw RFC call.')
		self.mprint('disconnect','Close connection with RFC server.')
		self.mprint('back','Return to previous menu.')
		return True
	
	def _back( self, parameters ):
		return False

	def run( self ):
		# TODO: Validate Targets and use of sapinfo
		om.out.debug('Inside stick plugin...')
		# Run stickShell
		try:
			core.sCore.validateTargets(self._targets)
		except dontRunException:
			return False		
		self.sh()
	
	def _connect(self, parameters):
		om.out.information('Trying to connect...')
		self._checkConnection()
		om.out.information('Sticked to remote RFC Server')
		return True
	
	def _disconnect(self, parameters):
		if self._conn.is_connected():
			self._conn.close()
		om.out.information('Disconnected.')
		return True

	def _call(self, parameters):
		'''
		This is the key method. It takes the specified funcname, parms and tables and performs the RFC call.
		Format:
			Export parameter:   "PAR_NAME", "RFCEXPORT", CHARTYPE, LEN, VALUE
			Import parameter:   "PAR_NAME", "RFCIMPORT", CHARTYPE, LEN
			Tables:						"TAB_NAME", LEN                               # Table can receive values only
		This is *really* beta...needs much more work.
		It doesn't support structures.
		'''
		# Clear result parameters
		self._resparams = []
		self._restables = []
		# Create iface
		ifc = iface(self._Options['funcname']['default'])
		# Create parameters
		for p in self._params:
			if p[1] == 'RFCEXPORT':
				ifc.addParm( parm(p[0], '', RFCEXPORT, CHARTYPE[p[2]], len=p[3], value=p[4]))
			elif p[1] == 'RFCIMPORT':
				self._resparams.append(parm(p[0], '', RFCIMPORT, CHARTYPE[p[2]], len=p[3]))
				ifc.addParm(self._resparams[len(self._resparams)-1])
			else:
				om.out.console('Parameter type ' + p[1] + ' not supported.')
				return False
				
		# Create tables
		for t in self._tables:
			self._restables.append(tab(t[0], '', t[1]))
			ifc.addParm(self._restables[len(self._restables)-1])
		try:
			if not saprfcutil.call_receive( self._conn, ifc ):
				raise sapytoException('Could not perform RFC call.')
		except TypeError, e:
			om.out.error('Error ocurred: ' + str(e))
			return False

		# Print results
		# Import Parameters
		for t in self._resparams:
			om.out.information("Contents of parameter " + t.name + ': ' + t.value)
		# Tables
		for t in self._restables:
			om.out.information("Contents of table " + t.name + ':')
			for line in t.value:
				om.out.information(line + '\n')
		return True
	
	def _view( self , parameters):
		'''
		List all configurable parameters of a plugin.
		'''
		self.mprint('Parameter','Value')
		self.mprint('=========','=====')
		for value in self._Options.keys():
			self.mprint(value,self._Options[value]['default'])
		return True
	
	def _set( self , parameters):
		'''
		Configure a value.
		'''
		if len( parameters ) < 2:
			om.out.console('Invalid call to set, please see the help:')
			self._help( ['set'] )
		else:
			param = parameters[0]
			value = parameters[1:]
			value = ' '.join( value )
			resVal = None
			if param in self._Options.keys():
				type = self._Options[param]['type']
				try:
					resVal = core.sCore.parseOption(type, value)
				except sapytoException:
					om.out.console( 'Parameter ' + param + ' must be of type ' +  type)
				else:
					# Special handling of params and tables
					if param == 'table' or param == 'param':
						self._Options[ param ]['default'].append(resVal)
					else:
						self._Options[ param ]['default'] = resVal
			else:	
				om.out.console( 'Unknown parameter.' )
		return True
		
	def _add(self, parameters):
		'''
		Adds parameters or tables.
		'''
		value = parameters[1:]
		value = ' '.join( value )
		if parameters[0] == 'param':
			try:
				resVal = core.sCore.parseOption('List', value)
			except sapytoException:
				om.out.console( 'Parameters must be of type List.' )
			self._params.append(resVal)
		elif parameters[0] == 'table':
			try:
				resVal = core.sCore.parseOption('List', value)
			except sapytoException:
				om.out.console( 'Parameters must be of type List.' )
			self._tables.append(resVal)
		else:
			om.out.console('Unknown item.')
	
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="funcname">\
				<default></default>\
				<desc>Function Name</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
			<Option name="param">\
				<default></default>\
				<desc>Function Parameter</desc>\
				<type>List</type>\
				<required>False</required>\
			</Option>\
			<Option name="table">\
				<default></default>\
				<desc>Function Table</desc>\
				<type>List</type>\
				<required>False</required>\
			</Option>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'funcname' in OptionList.keys(): 
			self._funcname = OptionList['funcname']
		if 'param' in OptionList.keys(): 
			self._params.append(OptionList['param'])
		if 'table' in OptionList.keys(): 
			self._tables.append(OptionList['table'])
