# -*- coding: latin-1 -*-
'''
checkGwmon.py

Copyright 2007 Mariano Nuez Di Croce @ CYBSEC

This file is part of sapyto, http://www.cybsec.com/EN/research/tools/sapyto.php

sapyto is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

sapyto is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sapyto; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
import core.sapytoCore as core
from core.basePlugins.basePlugin import basePlugin
import core.kb.knowledgeBase as kb
import core.output.outputManager as om
import saprfcutil
from saprfc import *
from core.exceptions.dontRunException import dontRunException

import socket
import struct

class checkGwmon(basePlugin):
	'''
	Detects if SAP Gateway allows remote connections.
	
	@author: Mariano Nuez Di Croce <mnunez@cybsec.com>
	'''
	def __init__( self):
		basePlugin.__init__( self )
		self._iface = None
		self._gwhost = 'Current target'
		self._gwserv = 'sapgw00'		
		
	def run( self ):
		try:
			core.sCore.validateTargets(self._targets)
		except dontRunException:
			return False		
		
		om.out.information('\tTrying to access Gateway Monitor...')
		
		if self._gwhost == 'Current target':
			self._gw = self._conn.ashost
		else:
			self._gw = self._gwhost
		
		# Create the socket to connect with GW
		self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		try:
			self._socket.connect((self._gw, socket.getservbyname(self._gwserv)))
		except socket.error, msg:
			om.out.information('Couldn\'t connect with remote Gateway.')
			return True
		
		# Define paqets to init connection with SAP GW
		# Send any command packet
		paq ='\x00'*3 + '\x88\x03\x09' + '\x12\xff\xff\x00\x00\x00\x00\x00\xc0\xfd\xd1' + '\x00'*123
		
		self._socket.send(paq)
		size = self._socket.recv(4)
		size = struct.unpack('!L', size)[0]
		if size != 0:		
			# Remote access is possible! You are soooo lucky!!! :P
			kb.kb.save('checkGwmon', 'remoteMonitor', True)
			# Receive the rest to clean buffers
			res = self._socket.recv(size)
			om.out.information('\tRemote access is possible.')
		else:
			kb.kb.save('checkGwmon', 'remoteMonitor', False)			
			om.out.information('\tRemote access is NOT possible.')
		self._socket.close()
		return True

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		sapyto/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
			<Option name="gwhost">\
				<default>Current target</default>\
				<desc>Gateway host</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
			<Option name="gwserv">\
				<default>sapgw00</default>\
				<desc>Gateway service</desc>\
				<type>String</type>\
				<required>True</required>\
			</Option>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		if 'gwhost' in OptionList.keys(): 
			self._gwhost = OptionList['gwhost']
		if 'gwserv' in OptionList.keys(): 
			self._gwserv = OptionList['gwserv']

	def setGwhost(self, gwhost):
		self._gwhost = gwhost
	
	def setGwserv(self, gwserv):
		self._service = gwserv
